package de.cliffbreak.core;

import de.cliffbreak.core.classes.StatusSign;
import de.cliffbreak.core.commands.CommandGM;
import de.cliffbreak.core.commands.CommandHub;
import de.cliffbreak.core.commands.CommandSign;
import de.cliffbreak.core.commands.CommandSpeed;
import de.cliffbreak.core.listeners.ChatListener;
import de.cliffbreak.core.listeners.PlayerJoinListener;
import de.cliffbreak.core.listeners.PlayerPortalListener;
import de.cliffbreak.core.listeners.PlayerQuitListener;
import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.configuration.file.YamlConfiguration;
import org.bukkit.plugin.java.JavaPlugin;

import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;

public class CliffbreakCore extends JavaPlugin {

    public ArrayList<StatusSign> statusSigns = new ArrayList<StatusSign>();

    public File cfgFile = new File("plugins/Core", "config.yml");
    public FileConfiguration cfg = YamlConfiguration.loadConfiguration(cfgFile);

    @SuppressWarnings("unchecked")
    @Override
    public void onEnable() {

        // Enable BungeeCord Messaging System
        this.getServer().getMessenger().registerOutgoingPluginChannel(this, "BungeeCord");

        getServer().getPluginManager().registerEvents(new PlayerJoinListener(this), this);
        getServer().getPluginManager().registerEvents(new PlayerQuitListener(), this);
        getServer().getPluginManager().registerEvents(new ChatListener(), this);
        getServer().getPluginManager().registerEvents(new PlayerPortalListener(this), this);
        this.getCommand("gm").setExecutor(new CommandGM());
        this.getCommand("speed").setExecutor(new CommandSpeed());
        this.getCommand("hub").setExecutor(new CommandHub(this));
        this.getCommand("sign").setExecutor(new CommandSign(this));

        if (cfg.getList("Signs") == null || cfg.getList("Signs").isEmpty())
            getLogger().info("No Signs registered. Skipping...");
        else
            for (Iterator i = cfg.getList("Signs").iterator(); i.hasNext(); ) {
                Map<String, Object> sign = (Map<String, Object>) i.next();
                getLogger().info("" + sign);

                Map<String, Object> locationMap = ((Map<String, Object>) sign.get("location"));
                Location location = new Location(
                        Bukkit.getServer().getWorld((String) locationMap.get("world")),
                        (double) (Integer) locationMap.get("x"),
                        (double) (Integer) locationMap.get("y"),
                        (double) (Integer) locationMap.get("z")
                );
                /* TODO: Add autoremoval
                if (location.getBlock() == null || (location.getBlock().getState() instanceof Sign)) {
                    getLogger().info("Nicht vorhandenes Schild wurde aus der Konfiguration entfernt. Position: " + location);
                }*/

                statusSigns.add(new StatusSign(location, (String) sign.get("host"), (Integer) sign.get("port"), (String) sign.get("name")));

                Bukkit.getScheduler().scheduleSyncRepeatingTask(this, new Runnable() {
                    @Override
                    public void run() {
                        for (StatusSign s : statusSigns) {
                            s.update();
                        }
                    }
                }, 0L, 5 * 20L);
            }
    }
}